import argparse
from .fetcher import fetch_all_publications
import json

def main():
    parser = argparse.ArgumentParser(description="Fetch publications from Google Scholar.")
    parser.add_argument("author_id", help="Google Scholar author ID")
    parser.add_argument("--max", type=int, default=None, help="Maximum number of publications to fetch")
    parser.add_argument("--sortby", type=str, default=None, help="How to sort publications (cited/pubdate)")
    args = parser.parse_args()

    publications = fetch_all_publications(args.author_id, args.max, args.sortby)
    print(json.dumps(publications, indent=4))
    print()

if __name__ == "__main__":
    main()



import requests
from bs4 import BeautifulSoup
import time

# Constants for headers, cookies, etc., can be defined here
HEADERS = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
    'user-agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Mobile Safari/537.36',
}

COOKIES = {
    'CONSENT': 'PENDING+300',
}

def fetch_all_publications(author_id, max_publications=None, sortby='cited'):
    all_publications = []
    start_index = 0
    page_size = 100
    total_fetched = 0

    while True:
        if max_publications is not None and total_fetched >= max_publications:
            break

        params = {
            'user': author_id,
            'oi': 'ao',
            'cstart': start_index,
            'pagesize': page_size,
            'sortby': sortby
        }

        response = requests.get('https://scholar.google.com/citations', params=params, cookies=COOKIES, headers=HEADERS)

        if response.status_code == 200:
            soup = BeautifulSoup(response.text, 'html.parser')
            publications = parse_publications(soup)
            if publications:
                if max_publications is not None:
                    remaining = max_publications - total_fetched
                    publications = publications[:remaining]

                all_publications.extend(publications)
                total_fetched += len(publications)

                if len(publications) < page_size:
                    break
                start_index += page_size
            else:
                break
        else:
            print(f"Failed to fetch data. Status code: {response.status_code}")
            break
        # Delay to prevent hitting rate limits
        time.sleep(1)  

    return all_publications

def parse_publications(soup):
    publications_list = []
    for row in soup.select('.gsc_a_tr'):
        title_element = row.select_one('.gsc_a_at')
        title = title_element.text if title_element else "No title"
        year_element = row.select_one('.gsc_a_h')
        year = year_element.text if year_element else "No year"
        link = f'https://scholar.google.com{title_element["href"]}' if title_element else "No link"
        citation_element = row.select_one('.gsc_a_ac')
        citations = citation_element.text if citation_element and citation_element.text.strip() else "0"  # Check if text exists and is not empty

        publications_list.append({
            "title": title,
            "year": year,
            "link": link,
            "citations": citations,
        })

    return publications_list


import unittest
from scholarly_publications.fetcher import fetch_all_publications

class TestScholarlyPublications(unittest.TestCase):
    
    def test_fetch_publications_limit(self):
        """Test fetching a limited number of publications."""
        author_id = '6nOPl94AAAAJ'  # Example Google Scholar ID
        max_publications = 1
        publications = fetch_all_publications(author_id, max_publications)
        
        # Assert that the number of publications returned matches the limit
        self.assertEqual(len(publications), max_publications)

if __name__ == '__main__':
    unittest.main()


from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

version = '0.6'

setup(
    name='scholarly_publications',
    version=version,
    description='A tool to fetch scholarly publications from Google Scholar by author ID',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/ezefranca/scholarly_publications',
    author='Ezequiel França',
    author_email='ezequiel.franca@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    keywords='scholarly publications google scholar fetcher academia',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=[
        'requests',
        'beautifulsoup4',
    ],
    entry_points={
        'console_scripts': [
            'scholarly_publications=scholarly_publications.cli:main',
        ],
    },
    project_urls={
        'Bug Reports': 'https://github.com/ezefranca/scholarly_publications/issues',
        'Source': 'https://github.com/ezefranca/scholarly_publications',
    },
)